#!/usr/bin/env python
# -*- coding: utf-8 -*-
############################################################################
#
# MODULE:	multiproc_mapcalc.py
# AUTHOR(S):	Soeren Gebbert
#
# PURPOSE:	Run multiple r.mapcalc processes to check parallel disk performance
# COPYRIGHT:	(C) 2012 by the GRASS Development Team
#
#		This program is free software under the GNU General Public
#		License (version 2). Read the file COPYING that comes with GRASS
#		for details.
#
#############################################################################

#%module
#% description: Run multiple r.mapcalc processes to check parallel disk performance
#%end

#%option G_OPT_R_BASE
#%end

#%option
#% key: nprocs
#% type: integer
#% description: The number of r.mapcalc processes to run in parallel
#% required: no
#% multiple: no
#% answer: 2
#%end

#%option
#% key: size
#% type: integer
#% description: The number of raster cells in millions to use, e.g. 25 means 25.000.000 cells. A temporary region will be used.
#% required: no
#% multiple: no
#% answer: 25
#% options: 1,5,10,25,50,100,1000
#%end

from multiprocessing import Process
import subprocess
import os
import time
import grass.script as grass 

###############################################################################

def run_mapcalc(name):
    print "### sub process for map <%s> ###"%(name) 
    print 'ppid %i pid %i'% (os.getppid(), os.getpid())
    grass.run_command("r.mapcalc", expr=name + " = 1", overwrite=True)

###############################################################################
    
if __name__ == '__main__':
    options, flags = grass.parser()
    base = options["base"]
    nprocs = int(options["nprocs"])
    size = int(options["size"])
    
    # We use a temporary region for computation
    grass.use_temp_region()
    
    s = 0
    w = 0
    n = 1
    e = 1
    res = 1
    # Compute region extent and resolution
    if size == 1:
	n = 10
	e = 10
	res = 0.01
    elif size == 5:
	n = 25
	e = 20
	res = 0.01
    elif size == 10:
	n = 40
	e = 25
	res = 0.01
    elif size == 25:
	n = 50
	e = 50
	res = 0.01
    elif size == 50:
	n = 80
	e = 62.5
	res = 0.01
    elif size == 100:
	n = 10
	e = 10
	res = 0.001
    elif size == 1000:
	n = 40
	e = 25
	res = 0.001

    grass.run_command("g.region", flags="p", n=n, s=s, e=e, w=w, res=res)

    print '### main process ###'
    print 'process id:', os.getpid()

    t1 = time.time()
    procs = []
    name_list = []

    # Start the processes
    for i in range(nprocs):
        name = "%s_%i" % (base,i)
        name_list.append(name)
        procs.append(Process(target=run_mapcalc, args=(name,)))
        procs[i].start() 
        
    # Join the processes
    for i in range(nprocs):
        procs[i].join() 
    
    print "Sync disk"
    subprocess.call(["sync"])

    t2 = time.time()
    print "Time for processing: %f seconds"%(t2 - t1)
    
    # Clean up
    for name in name_list:
	grass.run_command("g.remove", rast=name)
    
    grass.del_temp_region()
